# -*- coding: utf-8 -*-
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4

# Copyright (c) 2012 dput authors
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
"""
Local Uploader implementation
"""

from dput.uploader import AbstractUploader
from dput.exceptions import UploadException
import dput.util
import os.path


class LocalUploadException(UploadException):
    """
    Thrown in the event of a problem installing to a local file.
    This is a subclass of :class:`dput.exceptions.UploadException`.
    """
    pass


class LocalUploader(AbstractUploader):
    """
    Provides an interface to "upload" files to the local filesystem. This
    is helpful when you're dputting to the same system you're currently on,
    and do not wish to use `scp` or `sftp` as the transport (which is totally
    understandable).
    """

    def initialize(self, **kwargs):
        """
        See :meth:`dput.uploader.AbstractUploader.initialize`
        """
        pass

    def upload_file(self, filename, upload_filename=None):
        """
        See :meth:`dput.uploader.AbstractUploader.upload_file`
        """

        whereto = self._config['incoming']
        whereto = os.path.expanduser(whereto)
        if "HOME" in os.environ:
            whereto = os.path.join(os.environ["HOME"], whereto)
        if upload_filename is not None:
            whereto = os.path.join(whereto, upload_filename)
        command = [
            "install",
            "--mode",
            "0644",
            filename,
            whereto
        ]
        (_, e, x) = dput.util.run_command(command)
        if x != 0:
            raise LocalUploadException("Command %s returned failure %d: %s" % (
                repr(command), x, e)
            )

    def shutdown(self):
        """
        See :meth:`dput.uploader.AbstractUploader.shutdown`
        """
        pass
